/*
	CoreHTTP - Embeddable Single-process TCP/IP HTTP Server
	Copyright (C) 2005-2007  Frank Yaul

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	File: common.c
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/*
	all changeable globals
*/
#define GLOBALS_DEFINED
#include "common.h"
int TIMEOUT, PORT[SETSIZE], BACKLOG;
char *ROOTDIR, *LOGFILE, *PAGE404, *DIRLIST, **DEFPAGE, **CGIBIN, **CGIEXT,
	*UNKNOWNTYPE, **CONTENTTYPE, **CONTENTEXT;
struct sprock_t *FIRSTSPROCK;
	
/*
	initializing them by parsing settings file
*/
void InitGlobals(int num, char **args) {
	char var[PATHSIZE], val[PATHSIZE], str[PATHSIZE];
	FILE* cfgfile;
	int porti, defpagei, cgibini, cgiexti, contenttypei, contentexti;
	
	if (num != 2 || !strcmp(args[1], "--help")) {
		printf("usage: %s [--help] CfgFileAbsPath\n", args[0]);
		exit(EXIT_FAILURE);
	}
	if ((cfgfile = fopen(args[1], "r")) == NULL) {
		printf("unable to open config file!\n");
		exit(EXIT_FAILURE);
	}
	
	TIMEOUT = BACKLOG = -1;
	memset(PORT, -1, SETSIZE * sizeof(int));
	ROOTDIR = (char *)malloc(PATHSIZE * sizeof(char));
	LOGFILE = (char *)malloc(PATHSIZE * sizeof(char));
	PAGE404 = (char *)malloc(PATHSIZE * sizeof(char));
	DIRLIST = (char *)malloc(PATHSIZE * sizeof(char));
	UNKNOWNTYPE = (char *)malloc(PATHSIZE * sizeof(char));
	DEFPAGE = (char **)malloc(SETSIZE * sizeof(char *));
	CGIEXT = (char **)malloc(SETSIZE * sizeof(char *));
	CGIBIN = (char **)malloc(SETSIZE * sizeof(char *));
	CONTENTEXT = (char **)malloc(SETSIZE * sizeof(char *));
	CONTENTTYPE = (char **)malloc(SETSIZE * sizeof(char *));
	for (defpagei = 0; defpagei < SETSIZE; defpagei++) {
		DEFPAGE[defpagei] = (char *)malloc(PATHSIZE * sizeof(char));
		DEFPAGE[defpagei][0] = '\0';
		CGIEXT[defpagei] = (char *)malloc(PATHSIZE * sizeof(char));
		CGIEXT[defpagei][0] = '\0';
		CGIBIN[defpagei] = (char *)malloc(PATHSIZE * sizeof(char));
		CGIBIN[defpagei][0] = '\0';
		CONTENTEXT[defpagei] = (char *)malloc(PATHSIZE * sizeof(char));
		CONTENTEXT[defpagei][0] = '\0';
		CONTENTTYPE[defpagei] = (char *)malloc(PATHSIZE * sizeof(char));
		CONTENTTYPE[defpagei][0] = '\0';
	}
	ROOTDIR[0] = PAGE404[0] = LOGFILE[0] = '\0';
	porti = defpagei = cgiexti = cgibini = contenttypei = contentexti = 0;
	FIRSTSPROCK = NULL;
	
	while (feof(cfgfile) == 0) {
		if (fscanf(cfgfile, "%" PATHSIZE_S "[^\n]\n", str) < 1)
			continue;
		if (sscanf(str, "%s%*[ \t\n]%s", var, val) < 2)
			continue;
			
		/* integers */
		if (!strcmp(var, "TIMEOUT")) 
			TIMEOUT = (int)strtol(val, NULL, 0);
		else if (!strcmp(var, "PORT") && porti < SETSIZE) 
			PORT[porti++] = (int)strtol(val, NULL, 0);
		else if (!strcmp(var, "BACKLOG")) 
			BACKLOG = (int)strtol(val, NULL, 0);
		
		/* strings */
		sscanf(val, "\"%[^\"]\"", str);
		if (!strcmp(var, "ROOTDIR")) 
			realpath(str, ROOTDIR);
		else if (!strcmp(var, "DEFPAGE") && defpagei < SETSIZE) 
			strcpy(DEFPAGE[defpagei++], str);
		else if (!strcmp(var, "PAGE404")) 
			strcpy(PAGE404, str);
		else if (!strcmp(var, "LOGFILE"))
			strcpy(LOGFILE, str);
		else if (!strcmp(var, "CGIEXT") && cgiexti < SETSIZE)
			strcpy(CGIEXT[cgiexti++], str);
		else if (!strcmp(var, "CGIBIN") && cgibini < SETSIZE)
			strcpy(CGIBIN[cgibini++], str);
		else if (!strcmp(var, "CONTENTEXT") && contentexti < SETSIZE)
			strcpy(CONTENTEXT[contentexti++], str);
		else if (!strcmp(var, "CONTENTTYPE") && contenttypei < SETSIZE)
			strcpy(CONTENTTYPE[contenttypei++], str);
		else if (!strcmp(var, "DIRLIST"))
			strcpy(DIRLIST, str);
		else if (!strcmp(var, "UNKNOWNTYPE"))
			strcpy(UNKNOWNTYPE, str);
	}
	
	if (TIMEOUT == -1 || PORT[0] == -1 || BACKLOG == -1 || ROOTDIR == NULL
		|| ROOTDIR[0] == '\0' || DEFPAGE[0][0] == '\0' 
		|| PAGE404[0] == '\0' || LOGFILE[0] == '\0'
		|| cgibini != cgiexti || contenttypei != contentexti) {
		printf("bad config file!\n");
		exit(EXIT_FAILURE);
	}
	fclose(cfgfile);
}
