/*
	CoreHTTP - Embeddable Single-process TCP/IP HTTP Server
	Copyright (C) 2005-2007  Frank Yaul

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	File: handler.c
*/

#include "handler.h"

/* signal handler: everything is signal safe since we just exit .. */
void HandleExit(int signum) {
	char buffer[BUFSIZE], line[PATHSIZE];
	strcpy(buffer, "terminating... dumping existing sprockets ...\n");
	
	while (FIRSTSPROCK != NULL) {
		if (FIRSTSPROCK->func == CLNT)
			sprintf(line, "%-7d CLNT %s requesting %s\n", 
				FIRSTSPROCK->fd, 
				FIRSTSPROCK->ipaddr, FIRSTSPROCK->url);
		else if (FIRSTSPROCK->func == SERV)
			sprintf(line, "%-7d SERV\n", FIRSTSPROCK->fd);
		else if (FIRSTSPROCK->func == HTTP)
			sprintf(line, "%-7d HTTP\n", FIRSTSPROCK->fd); 
		strcat(buffer, line);
		RemoveSprock(FIRSTSPROCK, &FIRSTSPROCK);
	} 
		
	HandleLog(buffer);
	exit(EXIT_SUCCESS);
}

/* signal handler: clean up all zombie processes */
void HandleChildProcs(int signum) {
	while (waitpid(-1, NULL, WNOHANG) > 0);
}

/* die with error */
void HandleError(char *errstr) {
	fprintf(logfile, "%ld FATAL: %s\n", (long)time(NULL), errstr);
	HandleExit(-1);
}

/* live with error */
void HandleProblem(char *errstr) {
	char buffer[PATHSIZE] = "PROBLEM: "; /* strlen = 9 */
	strncat(buffer, errstr, PATHSIZE - 9);
	HandleLog(buffer);
}

/* log something good */
void HandleLog(char *str) {
	fprintf(logfile, "%ld %s\n", (long)time(NULL), str);
	fflush(logfile);
}

/* initialize handlers */
int HandlerInit(void) {
	struct sigaction handler;

	if ((logfile = fopen(LOGFILE, "a")) == NULL) return -1;
		
	if (sigemptyset(&handler.sa_mask) < 0) return -1;
	handler.sa_flags = 0;
	
	handler.sa_handler = SIG_IGN;
	if (sigaction(SIGPIPE, &handler, NULL) < 0) return -1;
	
	handler.sa_handler = HandleChildProcs;
	if (sigaction(SIGCHLD, &handler, NULL) < 0) return -1;

	handler.sa_handler = HandleExit;
	if (sigaction(SIGINT, &handler, NULL) < 0) return -1;
	if (sigaction(SIGTERM, &handler, NULL) < 0) return -1;
	if (sigaction(SIGQUIT, &handler, NULL) < 0) return -1;
	
	return 0;
}
