/*
	CoreHTTP - Embeddable Single-process TCP/IP HTTP Server
	Copyright (C) 2005-2007  Frank Yaul

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	File: http.c
*/

#include "http.h"

/* create a file sprocket which processes file parentsprock->buffer */
struct sprock_t *HttpSprockMake(struct sprock_t *parentsprock) {
	struct sprock_t *sprocket;
	char req[PATHSIZE], url[PATHSIZE], status[PATHSIZE], temp[BUFSIZE],
		args[PATHSIZE], ctype[PATHSIZE], ext[PATHSIZE], cmd[PATHSIZE],
		asciinum[3] = {0}, *find;
	int i, c, start, end, pipefd[2];
	FILE *pipetoprog;
	
	cmd[0] = '\0';

	/* general sprocket stuff */
	if ((sprocket = (struct sprock_t *)
		malloc(sizeof(struct sprock_t))) == NULL) return NULL;
	memset(sprocket, 0, sizeof(*sprocket));
	sprocket->func = HTTP;
	sprocket->state = INIT;
	sprocket->lastio = -1;
	parentsprock->child = sprocket;
	sprocket->parent = parentsprock;
   
	/* parse the request in buffer */
	sscanf(parentsprock->buffer, 
		"%" PATHSIZE_S "[A-Za-z] %" PATHSIZE_S "s%*[ \t\n]", req, url);
	strncpy(sprocket->parent->url, url, PATHSIZE); /* for logging */
	status[0] = '\0';
	sprintf(temp, "%s/%s", ROOTDIR, url);
	strncpy(url, temp, PATHSIZE);

	/* uppercase request */
	for (i = 0; req[i] != '\0'; i++)
		req[i] = toupper(req[i]);

	if (strcmp(req, "GET") && strcmp(req, "POST") && strcmp(req, "HEAD"))
		strcpy(status, "501 Not Implemented");

	/* de-escape it, turn %xx into a character */
	i = 0;
	while (url[i]) {
		if (url[i] == '+') url[i] = ' ';
		else if (url[i] == '%') {
			asciinum[0] = url[i + 1];
			asciinum[1] = url[i + 2];
			url[i] = strtol(asciinum, NULL, 16);
			c = i + 1;
			do { url[c] = url[c + 2]; } while (url[2 + c++]);
		}
		i++;
	}

	/* escape the url for " and \ since we use it in popen */
	for (i = 0; i < PATHSIZE; i++) {
		if (url[i] == '\0') break;
		else if (url[i] == '\\' || url[i] == '\"' || url[i] == '\'') {
			find = url + i;
			strcpy(temp, find);
			*find = '\\';
			*(find+1) = '\0';
			strcat(url, temp);
			i++;
		}
	}

	/* get split off urlencoded variables */
	find = strchr(url, '?');
	if (find == NULL) args[0] = '\0';
	else {
		*find = '\0';
		strcpy(args, find + 1);
	}
	/* TODO Note if we make HttpPostVars
		we might need to concatenate the variable strings. */
		
		
	/* if url is trying to traverse directories above root, drop it */
	realpath(url, temp);
	if (temp == NULL || strncmp(ROOTDIR, temp, strlen(ROOTDIR))) {
		sprintf(sprocket->buffer,
			"HTTP/1.1 403 Forbidden\r\n"
			"Server: " VERSION "\r\n"
			"Connection: close\r\n\r\n");
		sprocket->fd = -1;
		return sprocket;
	}
	
	/* inferred actions like 404 page, index.*, dir listing ... */      
	switch (HttpFileStat(url)) {
	default:
	case -1: /* not found */
		strcpy(status, "404 Not found");
		strcpy(url, PAGE404);
		break;
	case 0: /* is a directory */
		if (url[strlen(url) - 1] != '/') {
			find = url + strlen(ROOTDIR);
			sprintf(sprocket->buffer, 
				"HTTP/1.1 303 See Other\r\n"
				"Server: " VERSION "\r\n"
				"Location: %s/\r\n"
				"Connection: close\r\n\r\n",
				find);
			sprocket->fd = -1;
			return sprocket;
		}
		
		for (i = c = 0; DEFPAGE[i][0] != '\0' && i < SETSIZE; i++) {
			sprintf(temp, "%s/%s", url, DEFPAGE[i]);
			if (HttpFileStat(temp) == 1) {
				c = 1;
				strcpy(url, temp);
				break;
			}
		}
		if (c == 0)  { /* TODO our dirlist perl script takes the path
				of the dir as the arg. the way we do cgi
				right now is scipt.pl?arg turns into
				commandprompt> ./script.pl arg. obviously
				when urlencode is implemented correctly this
				must be changed. */
			strcpy(args, url);
			strcpy(url, DIRLIST);
			break;
		}
		/* fall through with index. break out with dirlist */
	case 1: /* is a file, do nothing */
		break;
	}

	/* get file extension */
	end = strlen(url) - 1;
	/* strrcspn, bah */
	for (start = end; start >= 0 
		&& url[start] != '.' && url[start] != '/'; start--);
	if (start == 0 || url[start] == '/') ext[0] = '\0';
	else strncpy(ext, url + start + 1, end - start + 1);

	/* get content type */
	ctype[0] = '\0';
	for (i = 0; CONTENTEXT[i][0] != '\0' && i < SETSIZE; i++)
		if (!strcmp(ext, CONTENTEXT[i])) {
			strcpy(ctype, CONTENTTYPE[i]);
			break;
		}
	if (ctype[0] == '\0') strcpy(ctype, UNKNOWNTYPE);

	/* if there isnt a command see if we need one for dynamic content */
	if (cmd[0] == '\0') 
		for (i = 0; CGIEXT[i][0] != '\0' && i < SETSIZE; i++)
			if (!strcmp(ext, CGIEXT[i])) {
				sprintf(cmd, "%s \"%s\" \"%s\"",
					CGIBIN[i], url, args);
				strcpy(ctype, "text/html");
				break;
			}

	/* handle type of request - header only, static, or dynamic content */
	if (!strcmp(req, "HEAD")) {
		sprocket->fd = -1;
	} else if (cmd[0] != '\0') { /* if its dynamic content */
		pipe(pipefd); /* make pipe then fork */
		c = fork();
		if (c > 0) { /* original, keep going */
			close(pipefd[1]); /* no need to write */
			sprocket->fd = pipefd[0];
			SetNonBlock(sprocket->fd);
		} else if (c == 0) { /* child, popen */
			close(pipefd[0]); /* no need to read */
			pipetoprog = popen(cmd, "r");
			/* fread should be non-blocking for this to exit fast
			when parent proc closes pipe */
			while ((i = fread(temp, 1, BUFSIZE, pipetoprog)) != 0
				&& write(pipefd[1], temp, i) > 0);
			pclose(pipetoprog);
			close(pipefd[1]);
			exit(EXIT_SUCCESS); /* exit after done */
		} else { /* failed */
			RemoveSprock(sprocket, &FIRSTSPROCK);
			return NULL;
		}
	} else if ((sprocket->fd = open(url, O_NONBLOCK | O_RDONLY)) < 0) {
		/* else its static */
		RemoveSprock(sprocket, &FIRSTSPROCK);
		return NULL;
	}

	/* put header */
	if (status[0] == '\0') strcpy(status, "200 OK");
	sprintf(sprocket->buffer, 
		"HTTP/1.1 %s\r\n"
		"Server: " VERSION "\r\n"
		"Content-Type: %s\r\n"
		"Connection: close\r\n\r\n",
		status, ctype);
	return sprocket;
}

/* return -1 error, 0 isdir, 1 isfile */
int HttpFileStat(char *filename) {
	struct stat filestat;
	if (stat(filename, &filestat) < 0) return -1;
	if (S_ISDIR(filestat.st_mode)) return 0;
	return 1;
}
