/*
	CoreHTTP - Embeddable Single-process TCP/IP HTTP Server
	Copyright (C) 2005-2007  Frank Yaul

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	File: sprock.h
*/

#ifndef SPROCK_H
#define SPROCK_H

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include "common.h"

/*
	sprocket structures, functions
*/
enum sprockstate_t { INIT, RECV, SEND, IDLE, TERM };
enum sprockfunc_t { SERV, CLNT, HTTP };

/* create a linked list of these and handle the ones select() says are ready */
struct sprock_t {
	enum sprockstate_t state;
	enum sprockfunc_t func;
	time_t lastio;
	int fd, /* file descriptor */
		bpos, upos; /* bytes in buffer, handled offset */
	struct sprock_t *next, *prev, *parent, *child; /* linked list/tree */
	char buffer[BUFSIZE], 
		*ipaddr, url[PATHSIZE]; /* for logging purposes */
};

struct sprock_t *ServSprockMake(unsigned short port);
struct sprock_t *ClntSprockAccept(struct sprock_t *parentsprock);

int SprockBufSend(struct sprock_t *sprocket);
int SprockBufRecv(struct sprock_t *sprocket);

/* linked list functions - give them any element within the list as param */
struct sprock_t *FirstSprock(struct sprock_t *sprocket);
struct sprock_t *NextSprock(struct sprock_t *sprocket);
void AddSprock(struct sprock_t *sprocket, struct sprock_t **firstptr);
void RemoveSprock(struct sprock_t *sprocket, struct sprock_t **firstptr);

#endif
